package com.example.mobile_labs.api.people

import android.content.Context
import androidx.paging.ExperimentalPagingApi
import androidx.paging.LoadType
import androidx.paging.PagingState
import androidx.paging.RemoteMediator
import androidx.room.withTransaction
import com.example.mobile_labs.TheatreApplication
import com.example.mobile_labs.api.MyServerService
import com.example.mobile_labs.api.models.toPerson
import com.example.mobile_labs.database.AppDatabase
import com.example.mobile_labs.database.person.model.Person
import com.example.mobile_labs.database.person.repository.OfflinePersonRepository
import com.example.mobile_labs.database.remotekeys.model.RemoteKeyType
import com.example.mobile_labs.database.remotekeys.model.RemoteKeys
import com.example.mobile_labs.database.remotekeys.repository.OfflineRemoteKeyRepository
import retrofit2.HttpException
import java.io.IOException

@OptIn(ExperimentalPagingApi::class)
class PeopleRemoteMediator(
    private val service: MyServerService,
    private val dbPersonRepository: OfflinePersonRepository,
    private val dbRemoteKeyRepository: OfflineRemoteKeyRepository,
    private val database: AppDatabase
) : RemoteMediator<Int, Person>() {

    override suspend fun initialize(): InitializeAction {
        return InitializeAction.LAUNCH_INITIAL_REFRESH
    }

    override suspend fun load(
        loadType: LoadType,
        state: PagingState<Int, Person>
    ): MediatorResult {
        val page = when (loadType) {
            LoadType.REFRESH -> {
                val remoteKeys = getRemoteKeyClosestToCurrentPosition(state)
                remoteKeys?.nextKey?.minus(1) ?: 1
            }

            LoadType.PREPEND -> {
                val remoteKeys = getRemoteKeyForFirstItem(state)
                remoteKeys?.prevKey
                    ?: return MediatorResult.Success(endOfPaginationReached = remoteKeys != null)
            }

            LoadType.APPEND -> {
                val remoteKeys = getRemoteKeyForLastItem(state)
                remoteKeys?.nextKey
                    ?: return MediatorResult.Success(endOfPaginationReached = remoteKeys != null)
            }
        }

        try {
            val people = service.getPeople(page, state.config.pageSize).map { it.toPerson() }
            val endOfPaginationReached = people.isEmpty()
            database.withTransaction {
                if (loadType == LoadType.REFRESH) {
                    dbRemoteKeyRepository.deleteRemoteKey(RemoteKeyType.PERSON)
                    dbPersonRepository.clearPeople()
                }
                val prevKey = if (page == 1) null else page - 1
                val nextKey = if (endOfPaginationReached) null else page + 1
                val keys = people.map {
                    RemoteKeys(
                        entityId = it.uid !!,
                        type = RemoteKeyType.PERSON,
                        prevKey = prevKey,
                        nextKey = nextKey
                    )
                }

//                dbRemoteKeyRepository.deleteRemoteKey(RemoteKeyType.PERSON)
//                dbPersonRepository.clearPeople()
                dbRemoteKeyRepository.createRemoteKeys(keys)
                dbPersonRepository.insertPeople(people)
            }
            return MediatorResult.Success(endOfPaginationReached = endOfPaginationReached)
        } catch (exception: IOException) {
            return MediatorResult.Error(exception)
        } catch (exception: HttpException) {
            return MediatorResult.Error(exception)
        }
    }

    private suspend fun getRemoteKeyForLastItem(state: PagingState<Int, Person>): RemoteKeys? {
        return state.pages.lastOrNull { it.data.isNotEmpty() }?.data?.lastOrNull()
            ?.let { person ->
                dbRemoteKeyRepository.getAllRemoteKeys(person.uid !!, RemoteKeyType.PERSON)
            }
    }

    private suspend fun getRemoteKeyForFirstItem(state: PagingState<Int, Person>): RemoteKeys? {
        return state.pages.firstOrNull { it.data.isNotEmpty() }?.data?.firstOrNull()
            ?.let { person ->
                dbRemoteKeyRepository.getAllRemoteKeys(person.uid !!, RemoteKeyType.PERSON)
            }
    }

    private suspend fun getRemoteKeyClosestToCurrentPosition(
        state: PagingState<Int, Person>
    ): RemoteKeys? {
        return state.anchorPosition?.let { position ->
            state.closestItemToPosition(position)?.uid?.let { personUid ->
                dbRemoteKeyRepository.getAllRemoteKeys(personUid, RemoteKeyType.PERSON)
            }
        }
    }

}