package com.example.mobile_labs.ui.event.list

import android.content.res.Configuration
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material3.Button
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.NavController
import androidx.paging.PagingData
import androidx.paging.compose.LazyPagingItems
import androidx.paging.compose.collectAsLazyPagingItems
import androidx.paging.compose.itemContentType
import androidx.paging.compose.itemKey
import coil.compose.AsyncImage
import com.example.mobile_labs.R
import com.example.mobile_labs.database.event.model.EventWithPerformance
import com.example.mobile_labs.common.AppViewModelProvider
import com.example.mobile_labs.ui.navigation.Screen
import com.example.mobile_labs.ui.theme.Mobile_LabsTheme
import kotlinx.coroutines.flow.MutableStateFlow
import java.time.format.DateTimeFormatter

@Composable
fun EventList(
    navController: NavController,
    viewModel: EventListViewModel = viewModel(factory = AppViewModelProvider.Factory)
) {
    val eventListUiState = viewModel.eventListUiState.collectAsLazyPagingItems()

    Column(
        horizontalAlignment = Alignment.CenterHorizontally,
        modifier = Modifier
            .padding(all = 10.dp)) {
        EventList(
            eventList = eventListUiState,
            onClick = {uid : Int ->
                val route = Screen.PerformanceView.route.replace("{id}", uid.toString())
                navController.navigate(route)
            }
        )
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
private fun EventList(
    modifier: Modifier = Modifier,
    eventList: LazyPagingItems<EventWithPerformance>,
    onClick: (uid: Int) -> Unit
) {
    Column(
        modifier = modifier
    ) {
        if (eventList.itemCount == 0) {
            Text(
                text = stringResource(R.string.events_missing_description),
                textAlign = TextAlign.Center,
                style = MaterialTheme.typography.titleLarge
            )
        } else {
            LazyColumn(modifier = Modifier.padding(all = 10.dp)) {
                items(
                    count = eventList.itemCount,
                    key = eventList.itemKey(),
                    contentType = eventList.itemContentType(),
                ) { index ->
                    val event = eventList[index]
                    event?.let{
                        EventListItem(event = event, modifier = Modifier
                            .padding(vertical = 7.dp), onClick = onClick)
                    }
                }
            }
        }
    }
}

@Composable
private fun EventListItem(
    event: EventWithPerformance, modifier: Modifier = Modifier,
    onClick: (uid: Int) -> Unit
) {
    Card(
        modifier = modifier.fillMaxWidth(),
        elevation = CardDefaults.cardElevation(defaultElevation = 2.dp)
    ) {
        Column(
            modifier = modifier.padding(all = 10.dp)
        ) {
            Text(
                text = event.event.date.format(DateTimeFormatter.ofPattern("dd.MM"))
            )
            AsyncImage(model = event.performance.imageURL, contentDescription = "")
            Button(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(all = 10.dp),
                onClick = { onClick(event.event.performanceId !!) }) {
                Text(event.performance.title)
            }
        }
    }
}

@Preview(name = "Light Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_NO)
@Preview(name = "Dark Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_YES)
@Composable
fun SchedulePreview() {
    Mobile_LabsTheme {
        Surface(
            color = MaterialTheme.colorScheme.background
        ) {
            EventList(eventList = MutableStateFlow(
                PagingData.empty<EventWithPerformance>()
            ).collectAsLazyPagingItems(),
                onClick = {})
        }
    }
}