package com.example.mobile_labs.ui.performance.list

import android.content.res.Configuration
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material3.Button
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.NavController
import androidx.paging.PagingData
import androidx.paging.compose.LazyPagingItems
import androidx.paging.compose.collectAsLazyPagingItems
import androidx.paging.compose.itemContentType
import androidx.paging.compose.itemKey
import coil.compose.AsyncImage
import com.example.mobile_labs.R
import com.example.mobile_labs.database.performance.model.Performance
import com.example.mobile_labs.common.AppViewModelProvider
import com.example.mobile_labs.ui.navigation.Screen
import com.example.mobile_labs.ui.theme.Mobile_LabsTheme
import kotlinx.coroutines.flow.MutableStateFlow

@Composable
fun PerformanceList(
    navController: NavController,
    viewModel: PerformanceListViewModel = viewModel(factory = AppViewModelProvider.Factory)
) {
    val performanceListUiState = viewModel.performanceListUiState.collectAsLazyPagingItems()

    Column(
        horizontalAlignment = Alignment.CenterHorizontally,
        modifier = Modifier
            .padding(all = 10.dp)) {
        PerformanceList(
            performanceList = performanceListUiState,
            onClick = {uid : Int ->
                val route = Screen.PerformanceView.route.replace("{id}", uid.toString())
                navController.navigate(route)
            }
        )
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
private fun PerformanceList(
    modifier: Modifier = Modifier,
    performanceList: LazyPagingItems<Performance>,
    onClick: (uid: Int) -> Unit
) {
    Column(
        modifier = modifier
    ) {
        if (performanceList.itemCount == 0) {
            Text(
                text = stringResource(R.string.performance_missing_description),
                textAlign = TextAlign.Center,
                style = MaterialTheme.typography.titleLarge
            )
        } else {
            LazyColumn(modifier = Modifier.padding(all = 10.dp)) {
                items(
                    count = performanceList.itemCount,
                    key = performanceList.itemKey(),
                    contentType = performanceList.itemContentType()
                ) { index ->
                    val performance = performanceList[index]
                    performance?.let {
                        PerformanceListItem(performance = performance, modifier = Modifier
                            .padding(vertical = 7.dp), onClick = onClick)
                    }
                }
            }
        }
    }
}

@Composable
private fun PerformanceListItem(
    performance: Performance, modifier: Modifier = Modifier,
    onClick: (uid: Int) -> Unit
) {
    Card(
        modifier = modifier.fillMaxWidth(),
        elevation = CardDefaults.cardElevation(defaultElevation = 2.dp)
    ) {
        Column(
            modifier = modifier.padding(all = 10.dp)
        ) {
            AsyncImage(model = performance.imageURL, contentDescription = "")
            Button(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(all = 10.dp),
                onClick = { onClick(performance.performance_uid !!) }) {
                Text(performance.title)
            }
        }
    }
}

@Preview(name = "Light Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_NO)
@Preview(name = "Dark Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_YES)
@Composable
fun SchedulePreview() {
    Mobile_LabsTheme {
        Surface(
            color = MaterialTheme.colorScheme.background
        ) {
            PerformanceList(
                performanceList = MutableStateFlow(
                    PagingData.empty<Performance>()
                ).collectAsLazyPagingItems(),
                onClick = {}
            )
        }
    }
}