package com.example.mobile_labs.ui.person.list

import android.content.res.Configuration
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.paging.PagingData
import androidx.paging.compose.LazyPagingItems
import androidx.paging.compose.collectAsLazyPagingItems
import androidx.paging.compose.itemContentType
import androidx.paging.compose.itemKey
import coil.compose.AsyncImage
import com.example.mobile_labs.R
import com.example.mobile_labs.database.person.model.Person
import com.example.mobile_labs.common.AppViewModelProvider
import com.example.mobile_labs.ui.theme.Mobile_LabsTheme
import kotlinx.coroutines.flow.MutableStateFlow

@Composable
fun PeopleList(
    viewModel: PeopleListViewModel = viewModel(factory = AppViewModelProvider.Factory)
) {
    val peopleListUiState = viewModel.personListUiState.collectAsLazyPagingItems()

    Column(
        horizontalAlignment = Alignment.CenterHorizontally,
        modifier = Modifier
            .padding(all = 10.dp)) {
        PeopleList(
            modifier = Modifier
                .fillMaxSize(),
            peopleList = peopleListUiState
        )
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
private fun PeopleList(
    modifier: Modifier = Modifier,
    peopleList: LazyPagingItems<Person>,
) {
    Column(
        modifier = modifier
    ) {
        if (peopleList.itemCount == 0) {
            Text(
                text = stringResource(R.string.people_missing_description),
                textAlign = TextAlign.Center,
                style = MaterialTheme.typography.titleLarge
            )
        } else {
            LazyColumn(modifier = Modifier.padding(all = 10.dp)) {
                items(
                    count = peopleList.itemCount,
                    key = peopleList.itemKey(),
                    contentType = peopleList.itemContentType()
                ) { index ->
                    val person = peopleList[index]
                    person?.let {
                        PeopleListItem(person = person, modifier = Modifier.padding(10.dp))
                    }
                }
            }
        }
    }
}

@Composable
private fun PeopleListItem(
    person: Person, modifier: Modifier = Modifier
) {
    Card(
        modifier = modifier.fillMaxWidth(),
        elevation = CardDefaults.cardElevation(defaultElevation = 2.dp)
    ) {
        Column(
            modifier = modifier.padding(all = 10.dp)
        ) {
            Row {
                AsyncImage(model = person.imageURL, contentDescription = "")
                Text(
                    text = String.format("%s %s", person.first_name, person.last_name),
                    textAlign = TextAlign.Center,
                    modifier = Modifier.padding(10.dp),
                    style = MaterialTheme.typography.displaySmall
                )
            }
        }
    }
}

@Preview(name = "Light Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_NO)
@Preview(name = "Dark Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_YES)
@Composable
fun PeopleListPreview() {
    Mobile_LabsTheme {
        Surface(
            color = MaterialTheme.colorScheme.background
        ) {
            PeopleList(peopleList = MutableStateFlow(
                PagingData.empty<Person>()
            ).collectAsLazyPagingItems())
        }
    }
}